import os
import logging, logging.handlers
#import splunk


LOG_DEFAULT_FMT = '%(asctime)s process:%(process)d thread:%(threadName)s %(levelname)s [%(name)s] [%(module)s:%(lineno)d] [%(funcName)s] %(message)s'
ITSI_MODELS_LOGGER = 'itsimodels'
ITSI_MODELS_LOG_FILE = 'itsi_content_packs_itsimodels.log'

try:
    SPLUNK_HOME = os.environ['SPLUNK_HOME']
except KeyError:
    SPLUNK_HOME = None

if SPLUNK_HOME is None:
    home_dir = os.path.expanduser("~")
    LOGGING_FILE_NAME = os.path.join(home_dir, ITSI_MODELS_LOG_FILE)

    print('-----> itsimodels log file is created at {}'.format(os.path.abspath(LOGGING_FILE_NAME)))

else:
    LOGGING_DEFAULT_CONFIG_FILE = os.path.join(SPLUNK_HOME, 'etc', 'log.cfg')
    LOGGING_LOCAL_CONFIG_FILE = os.path.join(SPLUNK_HOME, 'etc', 'log-local.cfg')

    LOGGING_STANZA_NAME = 'python'
    BASE_LOG_PATH = os.path.join('var', 'log', 'splunk')
    LOGGING_FILE_NAME = os.path.join(SPLUNK_HOME, BASE_LOG_PATH, ITSI_MODELS_LOG_FILE)

'''
cloned from splunk.setupSplunkLogger
'''
def setupSplunkLogger(baseLogger, defaultConfigFile, localConfigFile, loggingStanzaName, verbose=True):
    '''
    Takes the base logging.logger instance, and scaffolds the splunk logging namespace
    and sets up the logging levels as defined in the config files
    '''

    levels = getSplunkLoggingConfig(baseLogger, defaultConfigFile, localConfigFile, loggingStanzaName, verbose)

    for item in levels:
        loggerName = item[0]
        level = item[1]
        if hasattr(logging, level):
            logging.getLogger(loggerName).setLevel(getattr(logging, level))
        if verbose and (loggerName == "appender.python.maxFileSize" or loggerName == "appender.python.maxBackupIndex"):
            baseLogger.info('Python log rotation is not supported. Ignoring %s' % loggerName)


'''
cloned from splunk.getSplunkLoggingConfig
'''
def getSplunkLoggingConfig(baseLogger, defaultConfigFile, localConfigFile, loggingStanzaName, verbose):
    loggingLevels = []

    # read in config file and set logging levels
    if os.access(localConfigFile, os.R_OK):
        if verbose:
            baseLogger.info('Using local logging config file: %s' % localConfigFile)
        logConfig = open(localConfigFile, 'r')
    else:
        if verbose:
            baseLogger.info('Using default logging config file: %s' % defaultConfigFile)
        logConfig = open(defaultConfigFile, 'r')

    try:
        inStanza = False
        for line in logConfig:

            # strip comments
            line = line.strip()
            if '#' in line:
                line = line[:(line.index('#'))]

            # skip blank lines
            line = line.strip()
            if not line:
                continue

            # # # skip malformatted lines: stanza, key=value, or WTF?
            if line.startswith('['):
                if not line.endswith(']') or line.index(']') != (len(line) - 1):
                    continue
            elif '=' in line:
                key_test, value_test = line.split('=')
                if not key_test or not value_test:
                    continue
            else:
                continue

            # # # validation done, now we finally have parsing logic proper
            if not inStanza and line.startswith('[%s]' % loggingStanzaName):
                inStanza = True
                continue
            elif inStanza:
                if line.startswith('['):
                    break
                else:
                    name, level = line.split('=', 1)
                    if verbose:
                        baseLogger.info('Setting logger=%s level=%s' % (name.strip(), level.strip()))
                    loggingLevels.append((name.strip(), level.strip().upper()))
    except Exception as e:
        baseLogger.exception(e)
    finally:
        if logConfig: logConfig.close()

    return loggingLevels


def setup_logging():

    _logger = logging.getLogger(ITSI_MODELS_LOGGER)

    splunk_log_handler = logging.handlers.RotatingFileHandler(LOGGING_FILE_NAME, maxBytes=2500000, backupCount=5)
    splunk_log_handler.setFormatter(logging.Formatter(LOG_DEFAULT_FMT))
    _logger.addHandler(splunk_log_handler)

    if SPLUNK_HOME is not None:
        # splunk.setupSplunkLogger
        setupSplunkLogger(_logger, LOGGING_DEFAULT_CONFIG_FILE, LOGGING_LOCAL_CONFIG_FILE, LOGGING_STANZA_NAME)

    _logger.setLevel(logging.INFO)
    return _logger


logger = setup_logging()


from itsimodels.core.base_models import BaseModel, ChildModel
from itsimodels.core.compat import string_types
from itsimodels.core.fields import (
    BoolField,
    DictField,
    ForeignKey,
    ListField,
    NumberField,
    StringField,
    TypeField
)


class LaneOverlaySettings(ChildModel):
    is_enabled = StringField(default='no', alias='isEnabled')

    overlay_type = StringField(alias='overlayType')

    graph_color = StringField(alias='graphColor')

    graph_type = StringField(alias='graphType')

    entity_count = NumberField(alias='entityCount')

    selection_mode = StringField(alias='selectionMode')

    metric = StringField()

    selected_entities = ListField(alias='selectedEntities')

    search = StringField()


class DeepDiveLaneSettings(ChildModel):
    key = StringField(required=True, alias='id')

    data_gaps = StringField(default='connected', alias='dataGaps')

    data_model_stat_op = StringField(default='count', alias='dataModelStatOp')

    data_model_where_clause = StringField(default='', alias='dataModelWhereClause')

    data_model = DictField(string_types, alias='datamodelModel')

    distribution_stream_mode = StringField(default='quantile', alias='distributionStreamMode')

    entity_add_to_summary = StringField(default='yes', alias='entityAddToSummary')

    exclude_fields = ListField(string_types, alias='excludeFields')

    graph_color = StringField(default='', alias='graphColor')

    graph_series = ForeignKey('itsimodels.service.Kpi', alias='graphSeries')

    graph_type = StringField(default='line', alias='graphType')

    hide_graph = StringField(default='no', alias='hideGraph')

    kpi_add_to_summary = StringField(default='yes', alias='kpiAddToSummary')

    kpi_id = ForeignKey('itsimodels.service.Kpi', alias='kpiId')

    kpi_service_id = ForeignKey('itsimodels.service.Service', alias='kpiServiceId')

    kpi_service_title = StringField(alias='kpiServiceTitle')

    kpi_title = StringField(alias='kpiTitle')

    kpi_unit = StringField(default='', alias='kpiUnit')

    lane_overlay_settings = TypeField(LaneOverlaySettings, alias='laneOverlaySettingsModel')

    lane_size = StringField(default='small', alias='laneSize')

    lane_type = StringField(default='kpi', alias='laneType')

    overwrite_entity_title = StringField(default='no', alias='overwriteEntityTitle')

    overwrite_kpi_title = StringField(default='no', alias='overwriteKpiTitle')

    search = StringField(default='')

    search_source = StringField(alias='searchSource')

    subtitle = StringField(default='')

    threshold_indication_enabled = StringField(default='enabled', alias='thresholdIndicationEnabled')

    threshold_indication_type = StringField(default='stateIndication', alias='thresholdIndicationType')

    title = StringField(default='')

    vertical_axis_boundary_type = StringField(default='value', alias='verticalAxisBoundaryType')

    vertical_axis_scale = StringField(default='linear', alias='verticalAxisScale')

    vertical_axis_static_bounds = ListField(object, alias='verticalAxisStaticBounds')


class DeepDive(BaseModel):
    key = StringField(required=True, alias='_key')

    title = StringField(required=True)

    description = StringField(default='')

    earliest_time = StringField()

    focus_id = StringField()

    # interactable = BoolField(default=True)

    is_named = BoolField(default=True)

    lane_settings = ListField(DeepDiveLaneSettings, alias='lane_settings_collection')

    latest_time = StringField()

    topology_id = ForeignKey('itsimodels.service.Service')



from itsimodels.core.base_models import BaseModel, ChildModel
from itsimodels.core.fields import ForeignKey, ListField, StringField


GLOBAL_TEAM_KEY = 'default_itsi_security_group'


class TeamRelative(ChildModel):
    key = ForeignKey('itsimodels.team.Team', required=True, alias='_key')


class Team(BaseModel):
    key = StringField(required=True, alias='_key')

    title = StringField(required=True)

    description = StringField()

    inherit_from = ForeignKey('itsimodels.team.Team')

    children = ListField(TeamRelative)

    parents = ListField(TeamRelative)
